#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
@Time    : 8/18/20 2:05 PM
@Author  : Rodney Cheung
@File    : time.py
"""

import time


class TimeUtil:
    @staticmethod
    def format_seconds(seconds, time_format):
        """
        convert seconds to specified format
        Args:
            seconds: seconds
            time_format: specified time format,hour minute & second
            can be used

        Returns:
            formatted time str
        """
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        return str(time_format).format(h, m, s)

    @staticmethod
    def get_current_timestamp():
        """
        get current timestamp
        Returns:
            current timestamp
        """
        return time.time()

    @staticmethod
    def now(time_format: str) -> str:
        return time.strftime(time_format, time.localtime())



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : file.py
# Time       ：2020/10/14 10:37
# Author     ：Rodney Cheung
"""
from typing import AnyStr


class FileUtil:
    @staticmethod
    def read_file(path: str, bytes_num=-1) -> AnyStr:
        """
        read file content
        Args:
            path: file path
            bytes_num: read bytes,default is read all bytes
        Returns:
            file content
        """
        with open(path, 'rb') as f:
            return f.read(bytes_num)

    @staticmethod
    def last_line(file_path: str) -> str:
        """
        get last line of file,line separator will be ignored
        Args:
            file_path: file path
        Returns:
            content of last line,empty str if there is no content in file
        """
        with open(file_path) as fp:
            lines = fp.readlines()
            if len(lines) == 0:
                return ""
            return lines[-1].strip()

    @staticmethod
    def write_file(path: str, mode: str, data: AnyStr):
        with open(path, mode) as f:
            f.write(data)


# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : adb.py
# Time       ：2020/8/19 15:09
# Author     ：Rodney Cheung
"""
import os
import time
from typing import List, AnyStr

from wisbec.console.shell import exec_cmd
from wisbec.logging.log import Log


class AdbException(Exception):
    def __init__(self, err_info):
        self.err_info = err_info

    def __str__(self):
        return repr(self.err_info)


class Adb(object):
    adb_path: str = ''

    @classmethod
    def init(cls, adb_path: str):
        cls.adb_path = adb_path

    @staticmethod
    def devices(alive: bool = False) -> List[str]:
        """
        获取所有设备列表
        :param alive: 只显示在线的设备
        :return: 设备号数组
        """
        devices = list()
        ret_code, out, err = Adb.exec("devices")
        for line in out.split('\n')[1:]:
            splits = line.split()
            if len(splits) >= 2:
                device = splits[0]
                status = splits[1]
                if not alive or status == "device":
                    devices.append(device)
        return devices

    @staticmethod
    def wait_device(device_id: str, timeout):
        s = time.time()
        while time.time() - s <= timeout:
            list_devs = Adb.devices(alive=True)
            if device_id in list_devs:
                return True
            Log.warning('等待{}设备上线中...', device_id)
            time.sleep(1)
        pass

    # return: code, out, err
    @classmethod
    def exec(cls, *args) -> (int, AnyStr, AnyStr):
        """
        执行命令
        :param args: 命令
        :return: 输出结果
        """
        if cls.adb_path != '':
            args = [cls.adb_path] + list(args)
        else:
            args = ['adb'] + list(args)
        return exec_cmd(args)

    @staticmethod
    def shell(device_id: str, *args) -> (int, AnyStr, AnyStr):
        return Adb.exec('-s', device_id, 'shell', *args)

    @staticmethod
    def su_shell(device_id: str, *args) -> (int, AnyStr, AnyStr):
        code, out, err = Adb.shell(device_id, 'su', '-c', 'id', '-Z')
        if code != 0:
            su_cmd = '0'
        else:
            if 'magisk' in out:
                su_cmd = '-c'
            else:
                su_cmd = '0'
        return Adb.exec('-s', device_id, 'shell', 'su', su_cmd, *args)

    @staticmethod
    def top_app(device_id: str, sdk_level: int) -> str:
        code, out, err = Adb.shell(device_id,
                                   "dumpsys", "activity", "top", "|", "grep", "^TASK", "-A", "1")
        if sdk_level >= 26:
            current_app: str = out.strip().split(os.linesep)[-1].strip()
        else:
            current_app: str = out.strip().split(os.linesep)[1].strip()
        return current_app.split(' ')[1].split('/')[0]

    @staticmethod
    def screen_cap(device_id: str, save_path: str) -> bool:
        code, out, err = Adb.shell(device_id,
                                   "screencap", "-p", save_path)
        return out == '' and err == ''

    @staticmethod
    def push(device_id: str, src: str, dst: str) -> bool:
        code, out, err = Adb.exec('-s', device_id, 'push', src, dst)
        return code == 0

    @staticmethod
    def pull(device_id: str, src: str, dst: str) -> bool:
        code, out, err = Adb.exec('-s', device_id, 'pull', src, dst)
        return code == 0

    @staticmethod
    def install(device_id: str, apk_path: str) -> bool:
        code, out, err = Adb.exec('-s', device_id, 'install', apk_path)
        return 'Success' in out

    @staticmethod
    def id(device_id: str) -> int:
        code, out, err = Adb.exec('-s', device_id, 'id')
        s = out.find('uid=')
        e = out.find('(')
        return int(out[s + 4:e])

    # is_suc, msg
    @staticmethod
    def disable_verify(device_id: str) -> (bool, AnyStr):
        code, out, err = Adb.exec('-s', device_id, 'disable-verity')
        if err != '':
            return False, err
        return True, ''

    @staticmethod
    def get_sdk_level(device_id: str) -> int:
        code, out, err = Adb.shell(device_id, 'getprop', 'ro.build.version.sdk')
        if code != 0:
            raise AdbException('get device sdk level exception:{0}'.format(err))
        else:
            return int(out.strip())

    @staticmethod
    def get_system_packages(device_id: str) -> List[str]:
        code, out, err = Adb.shell(device_id, 'pm', 'list', 'packages', '-s')
        if code != 0:
            raise AdbException('get system packages exception:{0}'.format(err))
        else:
            packages: List[str] = out.split(os.linesep)
            system_packages = list()
            for package in packages:
                system_packages.append(package.split(':')[-1])
            return system_packages



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : zip.py
# Time       ：2020/8/19 15:50
# Author     ：Rodney Cheung
"""
import shutil


class ZipUtil:
    @staticmethod
    def zip_file(src_path, dst_path):
        shutil.make_archive(src_path, 'zip', dst_path)

    @staticmethod
    def unzip_file(src_zip, dst_dir=None, fmt=None):
        shutil.unpack_archive(src_zip, dst_dir, fmt)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

'''
Author: Sphantix Hang
Date: 2020-10-08 09:21:48
LastEditors: Sphantix Hang
LastEditTime: 2020-10-08 11:56:54
'''

import threading


class SingletonType(type):
    _instance_lock = threading.Lock()

    def __call__(cls, *args, **kwargs):
        if not hasattr(cls, "_instance"):
            with SingletonType._instance_lock:
                if not hasattr(cls, "_instance"):
                    cls._instance = super(
                        SingletonType, cls).__call__(*args, **kwargs)
        return cls._instance



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
Author: Sphantix Hang
Date: 2020-10-10 11:23:02
LastEditors: Sphantix Hang
LastEditTime: 2020-10-10 11:46:31
FilePath: /python-mod/src/wisbec/decorator/__init__.py
"""

from functools import wraps
from time import time


def timeit(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        start = time()
        result = f(*args, **kwargs)
        end = time()
        print('Elapsed time: {} sec'.format(end - start))
        return result

    return wrapper



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : hash.py
# Time       ：2020/10/13 17:19
# Author     ：Rodney Cheung
"""
import hashlib
import os
from typing import Optional

from wisbec.console import shell


class HashUtil:
    @staticmethod
    def get_file_md5(file_path: str) -> Optional[str]:
        if not os.path.isfile(file_path):
            return None
        md5hash = hashlib.md5()
        with open(file_path, 'rb') as f:
            while True:
                b = f.read(8096)
                if not b:
                    break
                md5hash.update(b)
        return md5hash.hexdigest()

    @staticmethod
    def calc_hash_name_by_openssl_x509(cacert_path: str) -> Optional[str]:
        args = ['openssl', 'x509', '-subject_hash_old', '-in', cacert_path]
        code, out, err = shell.exec_cmd(args)
        if code != 0:
            return None
        return '{}.0'.format(out.split(os.linesep)[0])



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : filesystem.py
# Time       ：2020/8/19 15:09
# Author     ：Rodney Cheung
"""
import os
import shutil
import stat
from typing import List


class FilesystemUtil:
    @staticmethod
    def is_directory_exist(dir_path: str) -> bool:
        """
        symbolic link or dir
        Args:
            dir_path: dir path

        Returns:

        """
        return os.path.isdir(dir_path)

    @staticmethod
    def is_file_exist(dir_path: str) -> bool:
        """
        symbolic link or file
        Args:
            dir_path:file path

        Returns:

        """
        return os.path.isfile(dir_path)

    @classmethod
    def create_directories(cls, dir_path: str):
        """
        create directory recursively
        Args:
            dir_path: dir path

        Returns:

        """
        if not cls.is_directory_exist(dir_path):
            os.makedirs(dir_path)

    @staticmethod
    def remove(file_path: str):
        if not os.path.exists(file_path):
            return
        else:
            if os.path.isfile(file_path):
                os.remove(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path, ignore_errors=True)

    @staticmethod
    def create_file(file_path: str):
        if os.path.exists(file_path):
            print("{0} already existed.".format(file_path))
            return

        f = open(file_path, 'w')
        f.close()

    @staticmethod
    def copy_file(src_full_path, dst_full_path):
        shutil.copy2(src_full_path, dst_full_path)

    @staticmethod
    def replace_extension(file_path: str, replacement: str):
        """
        replace file name's extension
        Args:
            file_path: file path
            replacement: target extension

        Returns:
            replaced file name
        """
        base_name, extension = os.path.splitext(file_path)
        if len(replacement) == 0:
            return base_name
        else:
            return base_name + '.' + replacement

    @classmethod
    def list_dir_recursively(cls, path: str, include_ext_name=None) -> list:
        """
        list files on path
        Args:
            path: target path
            include_ext_name: include extensions,if None,all files are listed

        Returns:
            file lists
        """
        files = os.listdir(path)
        res = list()
        for file in files:
            filepath = os.path.join(path, file)
            if os.path.isdir(filepath):
                res.extend(cls.list_dir_recursively(filepath, include_ext_name))
            else:
                if (include_ext_name is None) or (include_ext_name is not None and filepath.endswith(include_ext_name)):
                    res.append(filepath)
        return res

    @classmethod
    def list_dir(cls, path: str, depth=1, include_ext_name=None) -> list:
        files = os.listdir(path)
        res = list()
        if depth <= 0:
            return res
        for file in files:
            filepath = os.path.join(path, file)
            if os.path.isdir(filepath):
                depth -= 1
                res.extend(cls.list_dir(filepath, depth, include_ext_name))
                depth += 1
            else:
                if (include_ext_name is None) or (include_ext_name is not None and filepath.endswith(include_ext_name)):
                    res.append(filepath)
        return res

    @staticmethod
    def list_dirs_on_dir(path: str) -> List[str]:
        files = os.listdir(path)
        res = list()
        for file in files:
            filepath = os.path.join(path, file)
            if os.path.isdir(filepath):
                res.append(filepath)
        return res

    @staticmethod
    def add_executable(file_path: str):
        """
        equivalent of chmod +x file_path,posix supported only,do not call it on windows
        Args:
            file_path:

        Returns:

        """
        file_mode = os.stat(file_path).st_mode
        os.chmod(file_path, file_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)

    @staticmethod
    def remove_executable(file_path: str):
        """
        equivalent of chmod -x file_path,posix supported only,do not call it on windows
        Args:
            file_path:

        Returns:

        """
        file_mode = os.stat(file_path).st_mode
        os.chmod(file_path, file_mode & ~stat.S_IXUSR & ~stat.S_IXGRP & ~stat.S_IXOTH)
